@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -32,18 +32,73 @@
 #include "tbb_stddef.h"
 #include "tbb_machine.h"
 
-#if __TBB_EXCEPTIONS
-#include "cache_aligned_allocator.h"
-#endif /* __TBB_EXCEPTIONS */
+typedef struct ___itt_caller *__itt_caller;
 
 namespace tbb {
 
 class task;
 class task_list;
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
 class task_group_context;
-#endif /* __TBB_EXCEPTIONS */
+#endif /* __TBB_TASK_GROUP_CONTEXT */
+
+// MSVC does not allow taking the address of a member that was defined 
+// privately in task_base and made public in class task via a using declaration.
+#if _MSC_VER || (__GNUC__==3 && __GNUC_MINOR__<3)
+#define __TBB_TASK_BASE_ACCESS public
+#else
+#define __TBB_TASK_BASE_ACCESS private
+#endif
+
+namespace internal {
+
+    class allocate_additional_child_of_proxy: no_assign {
+        //! No longer used, but retained for binary layout compatibility.  Always NULL.
+        task* self;
+        task& parent;
+    public:
+        explicit allocate_additional_child_of_proxy( task& parent_ ) : self(NULL), parent(parent_) {}
+        task& __TBB_EXPORTED_METHOD allocate( size_t size ) const;
+        void __TBB_EXPORTED_METHOD free( task& ) const;
+    };
+
+}
+
+namespace interface5 {
+    namespace internal {
+        //! Base class for methods that became static in TBB 3.0.
+        /** TBB's evolution caused the "this" argument for several methods to become obsolete.
+            However, for backwards binary compatibility, the new methods need distinct names,
+            otherwise the One Definition Rule would be broken.  Hence the new methods are 
+            defined in this private base class, and then exposed in class task via 
+            using declarations. */
+        class task_base: tbb::internal::no_copy {
+        __TBB_TASK_BASE_ACCESS:
+            friend class tbb::task;
+
+            //! Schedule task for execution when a worker becomes available.
+            static void spawn( task& t );
+ 
+            //! Spawn multiple tasks and clear list.
+            static void spawn( task_list& list );
+
+            //! Like allocate_child, except that task's parent becomes "t", not this.
+            /** Typically used in conjunction with schedule_to_reexecute to implement while loops.
+               Atomically increments the reference count of t.parent() */
+            static tbb::internal::allocate_additional_child_of_proxy allocate_additional_child_of( task& t ) {
+                return tbb::internal::allocate_additional_child_of_proxy(t);
+            }
+
+            //! Destroy a task.
+            /** Usually, calling this method is unnecessary, because a task is
+                implicitly deleted after its execute() method runs.  However,
+                sometimes a task needs to be explicitly deallocated, such as
+                when a root task is used as the parent in spawn_and_wait_for_all. */
+            static void __TBB_EXPORTED_FUNC destroy( task& victim );
+        }; 
+    } // internal
+} // interface5
 
 //! @cond INTERNAL
 namespace internal {
@@ -62,16 +117,21 @@ namespace internal {
         //! Pure virtual destructor;
         //  Have to have it just to shut up overzealous compilation warnings
         virtual ~scheduler() = 0;
+#if __TBB_ARENA_PER_MASTER
+
+        //! For internal use only
+        virtual void enqueue( task& t, void* reserved ) = 0;
+#endif /* __TBB_ARENA_PER_MASTER */
     };
 
     //! A reference count
     /** Should always be non-negative.  A signed type is used so that underflow can be detected. */
-    typedef intptr reference_count;
+    typedef intptr_t reference_count;
 
     //! An id as used for specifying affinity.
     typedef unsigned short affinity_id;
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     struct context_list_node_t {
         context_list_node_t *my_prev,
                             *my_next;
@@ -84,7 +144,7 @@ namespace internal {
         task& __TBB_EXPORTED_METHOD allocate( size_t size ) const;
         void __TBB_EXPORTED_METHOD free( task& ) const;
     };
-#endif /* __TBB_EXCEPTIONS */
+#endif /* __TBB_TASK_GROUP_CONTEXT */
 
     class allocate_root_proxy: no_assign {
     public:
@@ -104,17 +164,6 @@ namespace internal {
         void __TBB_EXPORTED_METHOD free( task& ) const;
     };
 
-    class allocate_additional_child_of_proxy: no_assign {
-        task& self;
-        task& parent;
-    public:
-        allocate_additional_child_of_proxy( task& self_, task& parent_ ) : self(self_), parent(parent_) {}
-        task& __TBB_EXPORTED_METHOD allocate( size_t size ) const;
-        void __TBB_EXPORTED_METHOD free( task& ) const;
-    };
-
-    class task_group_base;
-
     //! Memory prefix to a task object.
     /** This class is internal to the library.
         Do not reference it directly, except within the library itself.
@@ -124,21 +173,21 @@ namespace internal {
     class task_prefix {
     private:
         friend class tbb::task;
+        friend class tbb::interface5::internal::task_base;
         friend class tbb::task_list;
         friend class internal::scheduler;
         friend class internal::allocate_root_proxy;
         friend class internal::allocate_child_proxy;
         friend class internal::allocate_continuation_proxy;
         friend class internal::allocate_additional_child_of_proxy;
-        friend class internal::task_group_base;
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
         //! Shared context that is used to communicate asynchronous state changes
         /** Currently it is used to broadcast cancellation requests generated both 
             by users and as the result of unhandled exceptions in the task::execute()
             methods. */
         task_group_context  *context;
-#endif /* __TBB_EXCEPTIONS */
+#endif /* __TBB_TASK_GROUP_CONTEXT */
         
         //! The scheduler that allocated the task, or NULL if the task is big.
         /** Small tasks are pooled by the scheduler that allocated the task.
@@ -164,7 +213,8 @@ namespace internal {
             In the "blocking style" of programming, this field is one more than the difference. */
         reference_count ref_count;
 
-        //! Scheduling depth
+        //! Obsolete. Used to be scheduling depth before TBB 2.2
+        /** Retained only for the sake of backward binary compatibility. **/
         int depth;
 
         //! A task::state_type, stored as a byte for compactness.
@@ -173,9 +223,10 @@ namespace internal {
 
         //! Miscellaneous state that is not directly visible to users, stored as a byte for compactness.
         /** 0x0 -> version 1.0 task
-            0x1 -> version 3.0 task
-            0x2 -> task_proxy
-            0x40 -> task has live ref_count */
+            0x1 -> version >=2.1 task
+            0x20 -> task_proxy
+            0x40 -> task has live ref_count
+            0x80 -> a stolen task */
         unsigned char extra_state;
 
         affinity_id affinity;
@@ -190,7 +241,7 @@ namespace internal {
 } // namespace internal
 //! @endcond
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
 
 #if TBB_USE_CAPTURED_EXCEPTION
     class tbb_exception;
@@ -221,8 +272,7 @@ namespace internal {
     When adding new members to task_group_context or changing types of existing ones, 
     update the size of both padding buffers (_leading_padding and _trailing_padding)
     appropriately. See also VERSIONING NOTE at the constructor definition below. **/
-class task_group_context : internal::no_copy
-{
+class task_group_context : internal::no_copy {
 private:
 #if TBB_USE_CAPTURED_EXCEPTION
     typedef tbb_exception exception_container_type;
@@ -244,7 +294,6 @@ public:
 
     enum traits_type {
         exact_exception = 0x0001ul << traits_offset,
-        no_cancellation = 0x0002ul << traits_offset,
         concurrent_wait = 0x0004ul << traits_offset,
 #if TBB_USE_CAPTURED_EXCEPTION
         default_traits = 0
@@ -268,12 +317,16 @@ private:
         its parent happens. Any context can be present in the list of one thread only. **/
     internal::context_list_node_t my_node;
 
+    //! Used to set and maintain stack stitching point for Intel Performance Tools.
+    __itt_caller itt_caller;
+
     //! Leading padding protecting accesses to frequently used members from false sharing.
     /** Read accesses to the field my_cancellation_requested are on the hot path inside
         the scheduler. This padding ensures that this field never shares the same cache 
         line with a local variable that is frequently written to. **/
     char _leading_padding[internal::NFS_MaxLineSize - 
-                    2 * sizeof(uintptr_t)- sizeof(void*) - sizeof(internal::context_list_node_t)];
+                    2 * sizeof(uintptr_t)- sizeof(void*) - sizeof(internal::context_list_node_t)
+                          - sizeof(__itt_caller)];
     
     //! Specifies whether cancellation was request for this task group.
     uintptr_t my_cancellation_requested;
@@ -379,30 +432,25 @@ private:
 
     static const kind_type binding_required = bound;
     static const kind_type binding_completed = kind_type(bound+1);
+    static const kind_type detached = kind_type(binding_completed+1);
+    static const kind_type dying = kind_type(detached+1);
 
     //! Checks if any of the ancestors has a cancellation request outstanding, 
     //! and propagates it back to descendants.
     void propagate_cancellation_from_ancestors ();
 
-    //! For debugging purposes only.
-    bool is_alive () { 
-#if TBB_USE_DEBUG
-        return my_version_and_traits != 0xDeadBeef;
-#else
-        return true;
-#endif /* TBB_USE_DEBUG */
-    }
 }; // class task_group_context
 
-#endif /* __TBB_EXCEPTIONS */
+#endif /* __TBB_TASK_GROUP_CONTEXT */
 
 //! Base class for user-defined tasks.
 /** @ingroup task_scheduling */
-class task: internal::no_copy {
+class task: __TBB_TASK_BASE_ACCESS interface5::internal::task_base {
+
     //! Set reference count
     void __TBB_EXPORTED_METHOD internal_set_ref_count( int count );
 
-    //! Decrement reference count and return true if non-zero.
+    //! Decrement reference count and return its new value.
     internal::reference_count __TBB_EXPORTED_METHOD internal_decrement_ref_count();
 
 protected:
@@ -429,7 +477,7 @@ public:
         //! task object is on free list, or is going to be put there, or was just taken off.
         freed,
         //! task to be recycled as continuation
-        recycle
+        recycle 
     };
 
     //------------------------------------------------------------------------
@@ -441,12 +489,12 @@ public:
         return internal::allocate_root_proxy();
     }
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     //! Returns proxy for overloaded new that allocates a root task associated with user supplied context.
     static internal::allocate_root_with_context_proxy allocate_root( task_group_context& ctx ) {
         return internal::allocate_root_with_context_proxy(ctx);
     }
-#endif /* __TBB_EXCEPTIONS */
+#endif /* __TBB_TASK_GROUP_CONTEXT */
 
     //! Returns proxy for overloaded new that allocates a continuation task of *this.
     /** The continuation's parent becomes the parent of *this. */
@@ -459,19 +507,20 @@ public:
         return *reinterpret_cast<internal::allocate_child_proxy*>(this);
     }
 
-    //! Like allocate_child, except that task's parent becomes "t", not this.
-    /** Typically used in conjunction with schedule_to_reexecute to implement while loops.
-        Atomically increments the reference count of t.parent() */
-    internal::allocate_additional_child_of_proxy allocate_additional_child_of( task& t ) {
-        return internal::allocate_additional_child_of_proxy(*this,t);
-    }
+    //! Define recommended static form via import from base class.
+    using task_base::allocate_additional_child_of;
 
+#if __TBB_DEPRECATED_TASK_INTERFACE
     //! Destroy a task.
     /** Usually, calling this method is unnecessary, because a task is
         implicitly deleted after its execute() method runs.  However,
         sometimes a task needs to be explicitly deallocated, such as
         when a root task is used as the parent in spawn_and_wait_for_all. */
-    void __TBB_EXPORTED_METHOD destroy( task& victim );
+    void __TBB_EXPORTED_METHOD destroy( task& t );
+#else /* !__TBB_DEPRECATED_TASK_INTERFACE */
+    //! Define recommended static form via import from base class.
+    using task_base::destroy;
+#endif /* !__TBB_DEPRECATED_TASK_INTERFACE */
 
     //------------------------------------------------------------------------
     // Recycling of tasks
@@ -490,7 +539,8 @@ public:
     }
 
     //! Recommended to use, safe variant of recycle_as_continuation
-    /** For safety, it requires additional increment of ref_count. */
+    /** For safety, it requires additional increment of ref_count.
+        With no decendants and ref_count of 1, it has the semantics of recycle_to_reexecute. */
     void recycle_as_safe_continuation() {
         __TBB_ASSERT( prefix().state==executing, "execute not running?" );
         prefix().state = recycle;
@@ -506,52 +556,25 @@ public:
         __TBB_ASSERT( new_parent.prefix().state!=freed, "parent already freed" );
         p.state = allocated;
         p.parent = &new_parent;
-        p.depth = new_parent.prefix().depth+1;
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
         p.context = new_parent.prefix().context;
-#endif /* __TBB_EXCEPTIONS */
+#endif /* __TBB_TASK_GROUP_CONTEXT */
     }
 
     //! Schedule this for reexecution after current execute() returns.
-    /** Requires that this.execute() be running. */
+    /** Made obsolete by recycle_as_safe_continuation; may become deprecated. */
     void recycle_to_reexecute() {
         __TBB_ASSERT( prefix().state==executing, "execute not running, or already recycled" );
         __TBB_ASSERT( prefix().ref_count==0, "no child tasks allowed when recycled for reexecution" );
         prefix().state = reexecute;
     }
 
-#if __TBB_TASK_DEQUE
     // All depth-related methods are obsolete, and are retained for the sake 
     // of backward source compatibility only
     intptr_t depth() const {return 0;}
     void set_depth( intptr_t ) {}
     void add_to_depth( int ) {}
 
-#else /* !__TBB_TASK_DEQUE */
-    //! A scheduling depth.
-    /** Guaranteed to be a signed integral type. */
-    typedef internal::intptr depth_type;
-
-    //! Scheduling depth
-    depth_type depth() const {return prefix().depth;}
-
-    //! Set scheduling depth to given value.
-    /** The depth must be non-negative */
-    void set_depth( depth_type new_depth ) {
-        __TBB_ASSERT( state()!=ready, "cannot change depth of ready task" );
-        __TBB_ASSERT( new_depth>=0, "depth cannot be negative" );
-        __TBB_ASSERT( new_depth==int(new_depth), "integer overflow error");
-        prefix().depth = int(new_depth);
-    }
-
-    //! Change scheduling depth by given amount.
-    /** The resulting depth must be non-negative. */
-    void add_to_depth( int delta ) {
-        __TBB_ASSERT( state()!=ready, "cannot change depth of ready task" );
-        __TBB_ASSERT( prefix().depth>=-delta, "depth cannot be negative" );
-        prefix().depth+=delta;
-    }
-#endif /* !__TBB_TASK_DEQUE */
 
     //------------------------------------------------------------------------
     // Spawning and blocking
@@ -573,7 +596,7 @@ public:
     }
 
     //! Atomically decrement reference count.  
-    /** Has release semanics. */  
+    /** Has release semantics. */  
     int decrement_ref_count() {
 #if TBB_USE_THREADING_TOOLS||TBB_USE_ASSERT
         return int(internal_decrement_ref_count());
@@ -582,27 +605,11 @@ public:
 #endif /* TBB_USE_THREADING_TOOLS||TBB_USE_ASSERT */
     }
 
-    //! Schedule task for execution when a worker becomes available.
-    /** After all children spawned so far finish their method task::execute,
-        their parent's method task::execute may start running.  Therefore, it
-        is important to ensure that at least one child has not completed until
-        the parent is ready to run. */
-    void spawn( task& child ) {
-#if !__TBB_RELAXED_OWNERSHIP
-        __TBB_ASSERT( is_owned_by_current_thread(), "'this' not owned by current thread" );
-#endif /* !__TBB_RELAXED_OWNERSHIP */
-        prefix().owner->spawn( child, child.prefix().next );
-    }
-
-    //! Spawn multiple tasks and clear list.
-    /** All of the tasks must be at the same depth. */
-    void spawn( task_list& list );
+    //! Define recommended static forms via import from base class.
+    using task_base::spawn;
 
     //! Similar to spawn followed by wait_for_all, but more efficient.
     void spawn_and_wait_for_all( task& child ) {
-#if !__TBB_RELAXED_OWNERSHIP
-        __TBB_ASSERT( is_owned_by_current_thread(), "'this' not owned by current thread" );
-#endif /* !__TBB_RELAXED_OWNERSHIP */
         prefix().owner->wait_for_all( *this, &child );
     }
 
@@ -610,12 +617,7 @@ public:
     void __TBB_EXPORTED_METHOD spawn_and_wait_for_all( task_list& list );
 
     //! Spawn task allocated by allocate_root, wait for it to complete, and deallocate it.
-    /** The thread that calls spawn_root_and_wait must be the same thread
-        that allocated the task. */
     static void spawn_root_and_wait( task& root ) {
-#if !__TBB_RELAXED_OWNERSHIP
-        __TBB_ASSERT( root.is_owned_by_current_thread(), "root not owned by current thread" );
-#endif /* !__TBB_RELAXED_OWNERSHIP */
         root.prefix().owner->spawn_root_and_wait( root, root.prefix().next );
     }
 
@@ -627,34 +629,30 @@ public:
     //! Wait for reference count to become one, and set reference count to zero.
     /** Works on tasks while waiting. */
     void wait_for_all() {
-#if !__TBB_RELAXED_OWNERSHIP
-        __TBB_ASSERT( is_owned_by_current_thread(), "'this' not owned by current thread" );
-#endif /* !__TBB_RELAXED_OWNERSHIP */
         prefix().owner->wait_for_all( *this, NULL );
     }
 
+#if __TBB_ARENA_PER_MASTER
+    //! Enqueue task for starvation-resistant execution.
+    static void enqueue( task& t ) {
+        t.prefix().owner->enqueue( t, NULL );
+    }
+
+#endif /* __TBB_ARENA_PER_MASTER */
     //! The innermost task being executed or destroyed by the current thread at the moment.
     static task& __TBB_EXPORTED_FUNC self();
 
     //! task on whose behalf this task is working, or NULL if this is a root.
     task* parent() const {return prefix().parent;}
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     //! Shared context that is used to communicate asynchronous state changes
     task_group_context* context() {return prefix().context;}
-#endif /* __TBB_EXCEPTIONS */   
+#endif /* __TBB_TASK_GROUP_CONTEXT */   
 
-    //! True if task is owned by different thread than thread that owns its parent.
+    //! True if task was stolen from the task pool of another thread.
     bool is_stolen_task() const {
-#if __TBB_PROVIDE_VIRTUAL_SCHEDULER
-        // The virtual scheduler directly identifies stolen tasks.
-        int es_virtual_steal = 4;
-        if(prefix().extra_state & es_virtual_steal)
-            return true;
-#endif /* TBB_PROVIDE_VIRTUAL_SCHEDULER */
-        internal::task_prefix& p = prefix();
-        internal::task_prefix& q = parent()->prefix();
-        return p.owner!=q.owner;
+        return (prefix().extra_state & 0x80)!=0;
     }
 
     //------------------------------------------------------------------------
@@ -667,13 +665,13 @@ public:
     //! The internal reference count.
     int ref_count() const {
 #if TBB_USE_ASSERT
-        internal::reference_count ref_count = prefix().ref_count;
-        __TBB_ASSERT( ref_count==int(ref_count), "integer overflow error");
+        internal::reference_count ref_count_ = prefix().ref_count;
+        __TBB_ASSERT( ref_count_==int(ref_count_), "integer overflow error");
 #endif
         return int(prefix().ref_count);
     }
 
-    //! True if this task is owned by the calling thread; false otherwise.
+    //! Obsolete, and only retained for the sake of backward compatibility. Always returns true.
     bool __TBB_EXPORTED_METHOD is_owned_by_current_thread() const;
 
     //------------------------------------------------------------------------
@@ -697,28 +695,27 @@ public:
         The default action does nothing. */
     virtual void __TBB_EXPORTED_METHOD note_affinity( affinity_id id );
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     //! Initiates cancellation of all tasks in this cancellation group and its subordinate groups.
     /** \return false if cancellation has already been requested, true otherwise. **/
     bool cancel_group_execution () { return prefix().context->cancel_group_execution(); }
 
     //! Returns true if the context received cancellation request.
     bool is_cancelled () const { return prefix().context->is_group_execution_cancelled(); }
-#endif /* __TBB_EXCEPTIONS */
+#endif /* __TBB_TASK_GROUP_CONTEXT */
 
 private:
+    friend class interface5::internal::task_base;
     friend class task_list;
     friend class internal::scheduler;
     friend class internal::allocate_root_proxy;
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     friend class internal::allocate_root_with_context_proxy;
-#endif /* __TBB_EXCEPTIONS */
+#endif /* __TBB_TASK_GROUP_CONTEXT */
     friend class internal::allocate_continuation_proxy;
     friend class internal::allocate_child_proxy;
     friend class internal::allocate_additional_child_of_proxy;
     
-    friend class internal::task_group_base;
-
     //! Get reference to corresponding task_prefix.
     /** Version tag prevents loader on Linux from using the wrong symbol in debug builds. **/
     internal::task_prefix& prefix( internal::version_tag* = NULL ) const {
@@ -742,6 +739,7 @@ private:
     task* first;
     task** next_ptr;
     friend class task;
+    friend class interface5::internal::task_base;
 public:
     //! Construct empty list
     task_list() : first(NULL), next_ptr(&first) {}
@@ -775,21 +773,19 @@ public:
     }
 };
 
-inline void task::spawn( task_list& list ) {
-#if !__TBB_RELAXED_OWNERSHIP
-    __TBB_ASSERT( is_owned_by_current_thread(), "'this' not owned by current thread" );
-#endif /* !__TBB_RELAXED_OWNERSHIP */
+inline void interface5::internal::task_base::spawn( task& t ) {
+    t.prefix().owner->spawn( t, t.prefix().next );
+}
+
+inline void interface5::internal::task_base::spawn( task_list& list ) {
     if( task* t = list.first ) {
-        prefix().owner->spawn( *t, *list.next_ptr );
+        t->prefix().owner->spawn( *t, *list.next_ptr );
         list.clear();
     }
 }
 
 inline void task::spawn_root_and_wait( task_list& root_list ) {
     if( task* t = root_list.first ) {
-#if !__TBB_RELAXED_OWNERSHIP
-        __TBB_ASSERT( t->is_owned_by_current_thread(), "'this' not owned by current thread" );
-#endif /* !__TBB_RELAXED_OWNERSHIP */
         t->prefix().owner->spawn_root_and_wait( *t, *root_list.next_ptr );
         root_list.clear();
     }
@@ -805,7 +801,7 @@ inline void operator delete( void* task, const tbb::internal::allocate_root_prox
     tbb::internal::allocate_root_proxy::free( *static_cast<tbb::task*>(task) );
 }
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
 inline void *operator new( size_t bytes, const tbb::internal::allocate_root_with_context_proxy& p ) {
     return &p.allocate(bytes);
 }
@@ -813,7 +809,7 @@ inline void *operator new( size_t bytes, const tbb::internal::allocate_root_with
 inline void operator delete( void* task, const tbb::internal::allocate_root_with_context_proxy& p ) {
     p.free( *static_cast<tbb::task*>(task) );
 }
-#endif /* __TBB_EXCEPTIONS */
+#endif /* __TBB_TASK_GROUP_CONTEXT */
 
 inline void *operator new( size_t bytes, const tbb::internal::allocate_continuation_proxy& p ) {
     return &p.allocate(bytes);
