@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -30,6 +30,7 @@
 #include "tbb/spin_mutex.h"
 #include "tbb/cache_aligned_allocator.h"
 #include "itt_notify.h"
+#include "semaphore.h"
 
 
 namespace tbb {
@@ -43,21 +44,33 @@ struct task_info {
     Token my_token;
     //! False until my_token is set.
     bool my_token_ready;
+    //! True if my_object is valid.
     bool is_valid;
+    //! Set to initial state (no object, no token)
+    void reset() {
+        my_object = NULL;
+        my_token = 0;
+        my_token_ready = false;
+        is_valid = false;
+    }
 };
-static task_info empty_info = {NULL, 0, false, false};
-
 //! A buffer of input items for a filter.
 /** Each item is a task_info, inserted into a position in the buffer corresponding to a Token. */
 class input_buffer {
     friend class tbb::internal::pipeline_root_task;
+    friend class tbb::filter;
     friend class tbb::thread_bound_filter;
+    friend class tbb::internal::stage_task;
+    friend class tbb::pipeline;
 
     typedef  Token  size_type;
 
     //! Array of deferred tasks that cannot yet start executing. 
     task_info* array;
 
+    //! for thread-bound filter, semaphore for waiting, NULL otherwise.
+    semaphore* my_sem;
+
     //! Size of array
     /** Always 0 or a power of 2 */
     size_type array_size;
@@ -77,7 +90,7 @@ class input_buffer {
     /** Must be a power of 2 */
     static const size_type initial_buffer_size = 4;
 
-    //! Used only for out of order buffer.
+    //! Used for out of order buffer, and for assigning my_token if is_ordered and my_token not already assigned
     Token high_token;
 
     //! True for ordered filter, false otherwise. 
@@ -85,14 +98,20 @@ class input_buffer {
 
     //! True for thread-bound filter, false otherwise. 
     bool is_bound;
+
+    void create_sema(size_t initial_tokens) { __TBB_ASSERT(!my_sem,NULL); my_sem = new internal::semaphore(initial_tokens); }
+    void free_sema() { __TBB_ASSERT(my_sem,NULL); delete my_sem; }
+    void sema_P() { __TBB_ASSERT(my_sem,NULL); my_sem->P(); }
+    void sema_V() { __TBB_ASSERT(my_sem,NULL); my_sem->V(); }
 public:
     //! Construct empty buffer.
     input_buffer( bool is_ordered_, bool is_bound_ ) : 
-            array(NULL), array_size(0),
+            array(NULL), my_sem(NULL), array_size(0),
             low_token(0), high_token(0), 
             is_ordered(is_ordered_), is_bound(is_bound_) {
         grow(initial_buffer_size);
         __TBB_ASSERT( array, NULL );
+        if(is_bound) create_sema(0);
     }
 
     //! Destroy the buffer.
@@ -100,34 +119,46 @@ public:
         __TBB_ASSERT( array, NULL );
         cache_aligned_allocator<task_info>().deallocate(array,array_size);
         poison_pointer( array );
+        if(my_sem) {
+            free_sema();
+            my_sem = NULL;
+        }
     }
 
     //! Put a token into the buffer.
     /** If task information was placed into buffer, returns true;
         otherwise returns false, informing the caller to create and spawn a task.
+        If input buffer owned by thread-bound filter and the item at
+        low_token was not valid, issue a V()
+        If the input_buffer is owned by a successor to a thread-bound filter,
+        the force_put parameter should be true to ensure the token is inserted
+        in the buffer.
     */
-    // Using template to avoid explicit dependency on stage_task
-    template<typename StageTask>
-    bool put_token( StageTask& putter ) {
+    bool put_token( task_info& info_, bool force_put = false ) {
         {
+            info_.is_valid = true;
             spin_mutex::scoped_lock lock( array_mutex );
             Token token;
+            bool was_empty = !array[low_token&(array_size-1)].is_valid;
             if( is_ordered ) {
-                if( !putter.my_token_ready ) {
-                    putter.my_token = high_token++;
-                    putter.my_token_ready = true;
+                if( !info_.my_token_ready ) {
+                    info_.my_token = high_token++;
+                    info_.my_token_ready = true;
                 }
-                token = putter.my_token;
+                token = info_.my_token;
             } else
                 token = high_token++;
             __TBB_ASSERT( (tokendiff_t)(token-low_token)>=0, NULL );
-            if( token!=low_token || is_bound ) {
+            if( token!=low_token || is_bound || force_put ) {
                 // Trying to put token that is beyond low_token.
                 // Need to wait until low_token catches up before dispatching.
                 if( token-low_token>=array_size ) 
                     grow( token-low_token+1 );
                 ITT_NOTIFY( sync_releasing, this );
-                putter.put_task_info(array[token&array_size-1]);
+                array[token&(array_size-1)] = info_;
+                if(was_empty && is_bound) {
+                    sema_V();
+                }
                 return true;
             }
         }
@@ -137,14 +168,19 @@ public:
     //! Note that processing of a token is finished.
     /** Fires up processing of the next token, if processing was deferred. */
     // Using template to avoid explicit dependency on stage_task
+    // this is only called for serial filters, and is the reason for the
+    // advance parameter in return_item (we're incrementing low_token here.)
+    // Non-TBF serial stages don't advance the token at the start because the presence
+    // of the current token in the buffer keeps another stage from being spawned.
     template<typename StageTask>
     void note_done( Token token, StageTask& spawner ) {
-        task_info wakee = empty_info;
+        task_info wakee;
+        wakee.reset();
         {
             spin_mutex::scoped_lock lock( array_mutex );
             if( !is_ordered || token==low_token ) {
                 // Wake the next task
-                task_info& item = array[++low_token & array_size-1];
+                task_info& item = array[++low_token & (array_size-1)];
                 ITT_NOTIFY( sync_acquired, this );
                 wakee = item;
                 item.is_valid = false;
@@ -154,12 +190,12 @@ public:
             spawner.spawn_stage_task(wakee);
     }
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     //! The method destroys all data in filters to prevent memory leaks
     void clear( filter* my_filter ) {
         long t=low_token;
         for( size_type i=0; i<array_size; ++i, ++t ){
-            task_info& temp = array[t&array_size-1];
+            task_info& temp = array[t&(array_size-1)];
             if (temp.is_valid ) {
                 my_filter->finalize(temp.my_object);
                 temp.is_valid = false;
@@ -168,9 +204,12 @@ public:
     }
 #endif
 
+    //! return an item, invalidate the queued item, but only advance if advance
+    //  advance == true for parallel filters.  If the filter is serial, leave the
+    // item in the buffer to keep another stage from being spawned.
     bool return_item(task_info& info, bool advance) {
         spin_mutex::scoped_lock lock( array_mutex );
-        task_info& item = array[low_token&array_size-1];
+        task_info& item = array[low_token&(array_size-1)];
         ITT_NOTIFY( sync_acquired, this );
         if( item.is_valid ) {
             info = item;
@@ -181,24 +220,8 @@ public:
         return false;
     }
 
-    void put_item( task_info& info ) {
-        info.is_valid = true;
-        spin_mutex::scoped_lock lock( array_mutex );
-        Token token;
-        if( is_ordered ) {
-            if( !info.my_token_ready ) {
-                info.my_token = high_token++;
-                info.my_token_ready = true;
-            }
-            token = info.my_token;
-        } else
-            token = high_token++;
-        __TBB_ASSERT( (tokendiff_t)(token-low_token)>=0, NULL );
-        if( token-low_token>=array_size ) 
-            grow( token-low_token+1 );
-        ITT_NOTIFY( sync_releasing, this );
-        array[token&array_size-1] = info;
-    }
+    //! true if the current low_token is valid.
+    bool has_item() { spin_mutex::scoped_lock lock(array_mutex); return array[low_token&(array_size -1)].is_valid; }
 };
 
 void input_buffer::grow( size_type minimum_size ) {
@@ -212,7 +235,7 @@ void input_buffer::grow( size_type minimum_size ) {
         new_array[i].is_valid = false;
     long t=low_token;
     for( size_type i=0; i<old_size; ++i, ++t )
-        new_array[t&new_size-1] = old_array[t&old_size-1];
+        new_array[t&(new_size-1)] = old_array[t&(old_size-1)];
     array = new_array;
     array_size = new_size;
     if( old_array )
@@ -226,15 +249,17 @@ private:
     filter* my_filter;  
     //! True if this task has not yet read the input.
     bool my_at_start;
+
 public:
     //! Construct stage_task for first stage in a pipeline.
     /** Such a stage has not read any input yet. */
     stage_task( pipeline& pipeline ) :
-        task_info(empty_info),
         my_pipeline(pipeline), 
         my_filter(pipeline.filter_list),
         my_at_start(true)
-    {}
+    {
+        task_info::reset();
+    }
     //! Construct stage_task for a subsequent stage in a pipeline.
     stage_task( pipeline& pipeline, filter* filter_, const task_info& info ) :
         task_info(info),
@@ -242,18 +267,24 @@ public:
         my_filter(filter_),
         my_at_start(false)
     {}
+    //! Roughly equivalent to the constructor of input stage task
+    void reset() {
+        task_info::reset();
+        my_filter = my_pipeline.filter_list;
+        my_at_start = true;
+    }
     //! The virtual task execution method
     /*override*/ task* execute();
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     ~stage_task()    
     {
         if (my_filter && my_object && (my_filter->my_filter_mode & filter::version_mask) >= __TBB_PIPELINE_VERSION(4)) {
-            __TBB_ASSERT(is_cancelled(), "Tryning to finalize the task that wasn't cancelled");
+            __TBB_ASSERT(is_cancelled(), "Trying to finalize the task that wasn't cancelled");
             my_filter->finalize(my_object);
             my_object = NULL;
         }
     }
-#endif // __TBB_EXCEPTIONS
+#endif // __TBB_TASK_GROUP_CONTEXT
     //! Creates and spawns stage_task from task_info
     void spawn_stage_task(const task_info& info)
     {
@@ -261,13 +292,6 @@ public:
                                 stage_task( my_pipeline, my_filter, info );
         spawn(*clone);
     }
-    //! Puts current task information
-    void put_task_info(task_info &where_to_put ) {
-        where_to_put.my_object = my_object;
-        where_to_put.my_token = my_token;
-        where_to_put.my_token_ready = my_token_ready;
-        where_to_put.is_valid = true;
-    }
 };
 
 task* stage_task::execute() {
@@ -284,9 +308,14 @@ task* stage_task::execute() {
                     if( my_pipeline.has_thread_bound_filters )
                         my_pipeline.token_counter++; // ideally, with relaxed semantics
                 }
-                ITT_NOTIFY( sync_releasing, &my_pipeline.input_tokens );
-                if( --my_pipeline.input_tokens>0 ) 
-                    spawn( *new( allocate_additional_child_of(*parent()) ) stage_task( my_pipeline ) );
+                if( !my_filter->next_filter_in_pipeline ) { // we're only filter in pipeline
+                    reset();
+                    goto process_another_stage;
+                } else {
+                    ITT_NOTIFY( sync_releasing, &my_pipeline.input_tokens );
+                    if( --my_pipeline.input_tokens>0 )
+                        spawn( *new( allocate_additional_child_of(*parent()) ) stage_task( my_pipeline ) );
+                }
             } else {
                 my_pipeline.end_of_input = true; 
                 return NULL;
@@ -306,7 +335,7 @@ task* stage_task::execute() {
                 my_pipeline.end_of_input = true; 
                 if( (my_filter->my_filter_mode & my_filter->version_mask) >= __TBB_PIPELINE_VERSION(5) ) {
                     if( my_pipeline.has_thread_bound_filters )
-                        my_pipeline.token_counter--;
+                        my_pipeline.token_counter--;  // fix token_counter
                 }
                 return NULL;
             }
@@ -317,7 +346,6 @@ task* stage_task::execute() {
         if( my_filter->is_serial() )
             my_filter->my_input_buffer->note_done(my_token, *this);
     }
-    task* next = NULL;
     my_filter = my_filter->next_filter_in_pipeline; 
     if( my_filter ) {
         // There is another filter to execute.
@@ -333,31 +361,36 @@ task* stage_task::execute() {
                         my_filter = my_filter->next_filter_in_pipeline;
                     } while( my_filter && my_filter->is_bound() );
                     // Check if there is an item ready to process
-                    if( my_filter && my_filter->my_input_buffer->return_item(*this, !my_filter->is_serial()) ) 
+                    if( my_filter && my_filter->my_input_buffer->return_item(*this, !my_filter->is_serial()))
                         goto process_another_stage;
                 } 
                 my_filter = NULL; // To prevent deleting my_object twice if exception occurs
                 return NULL;
             }
         }
-process_another_stage:
-        /* A semi-hackish way to reexecute the same task object immediately without spawning.
-           recycle_as_continuation marks the task for future execution,
-           and then 'this' pointer is returned to bypass spawning. */
-        recycle_as_continuation();
-        next = this;
     } else {
-        // Reached end of the pipe.  Inject a new token.
-        // The token must be injected before execute() returns, in order to prevent the
-        // parent's reference count from prematurely reaching 0.
-        set_depth( parent()->depth()+1 ); 
-        if( ++my_pipeline.input_tokens==1 ) {
-            ITT_NOTIFY( sync_acquired, &my_pipeline.input_tokens );
-            if( !my_pipeline.end_of_input && !my_pipeline.filter_list->is_bound() ) 
-                spawn( *new( allocate_additional_child_of(*parent()) ) stage_task( my_pipeline ) );
+        // Reached end of the pipe.
+        size_t ntokens_avail = ++my_pipeline.input_tokens;
+        if(my_pipeline.filter_list->is_bound() ) {
+            if(ntokens_avail == 1) {
+                my_pipeline.filter_list->my_input_buffer->sema_V();
+            }
+            return NULL;
         }
+        if( ntokens_avail>1  // Only recycle if there is one available token
+                || my_pipeline.end_of_input ) {
+            return NULL; // No need to recycle for new input
+        }
+        ITT_NOTIFY( sync_acquired, &my_pipeline.input_tokens );
+        // Recycle as an input stage task.
+        reset();
     }
-    return next;
+process_another_stage:
+    /* A semi-hackish way to reexecute the same task object immediately without spawning.
+       recycle_as_continuation marks the task for future execution,
+       and then 'this' pointer is returned to bypass spawning. */
+    recycle_as_continuation();
+    return this;
 }
 
 class pipeline_root_task: public task {
@@ -380,10 +413,10 @@ class pipeline_root_task: public task {
             while( current_filter ) {
                 __TBB_ASSERT( !current_filter->is_bound(), "filter is thread-bound?" );
                 __TBB_ASSERT( current_filter->prev_filter_in_pipeline->is_bound(), "previous filter is not thread-bound?" );
-                if( !my_pipeline.end_of_input
-                    || (tokendiff_t)(my_pipeline.token_counter - current_filter->my_input_buffer->low_token) > 0 )
+                if( !my_pipeline.end_of_input || current_filter->has_more_work())
                 {
-                    task_info info = empty_info;
+                    task_info info;
+                    info.reset();
                     if( current_filter->my_input_buffer->return_item(info, !current_filter->is_serial()) ) {
                         set_ref_count(1);
                         recycle_as_continuation();
@@ -404,7 +437,7 @@ class pipeline_root_task: public task {
                     first_suitable_filter = first_suitable_filter->next_segment;
                     current_filter = first_suitable_filter; 
                 }
-            } /* end of while */
+            } /* while( current_filter ) */
             return NULL;
         } else { 
             if( !my_pipeline.end_of_input ) {
@@ -450,7 +483,7 @@ public:
         my_pipeline(_pipeline)
     {}
     ~pipeline_cleaner(){
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
         if (my_pipeline.end_counter->is_cancelled()) // Pipeline was cancelled
             my_pipeline.clear_filters(); 
 #endif
@@ -464,7 +497,7 @@ void pipeline::inject_token( task& ) {
     __TBB_ASSERT(0,"illegal call to inject_token");
 }
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
 void pipeline::clear_filters() {
     for( filter* f = filter_list; f; f = f->next_filter_in_pipeline ) {
         if ((f->my_filter_mode & filter::version_mask) >= __TBB_PIPELINE_VERSION(4))
@@ -553,6 +586,9 @@ void pipeline::add_filter( filter& filter_ ) {
 }
 
 void pipeline::remove_filter( filter& filter_ ) {
+    __TBB_ASSERT( filter_.prev_filter_in_pipeline!=filter::not_in_pipeline(), "filter not part of pipeline" );
+    __TBB_ASSERT( filter_.next_filter_in_pipeline!=filter::not_in_pipeline(), "filter not part of pipeline" );
+    __TBB_ASSERT( !end_counter, "invocation of remove_filter on running pipeline" );
     if (&filter_ == filter_list) 
         filter_list = filter_.next_filter_in_pipeline;
     else {
@@ -576,46 +612,65 @@ void pipeline::remove_filter( filter& filter_ ) {
 }
 
 void pipeline::run( size_t max_number_of_live_tokens
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
     , tbb::task_group_context& context
 #endif
     ) {
     __TBB_ASSERT( max_number_of_live_tokens>0, "pipeline::run must have at least one token" );
     __TBB_ASSERT( !end_counter, "pipeline already running?" );
     if( filter_list ) {
-        if( filter_list->next_filter_in_pipeline || !filter_list->is_serial() ) {
-            internal::pipeline_cleaner my_pipeline_cleaner(*this);
-            end_of_input = false;
-#if __TBB_EXCEPTIONS            
-            end_counter = new( task::allocate_root(context) ) internal::pipeline_root_task( *this );
-#else
-            end_counter = new( task::allocate_root() ) internal::pipeline_root_task( *this );
-#endif
-            input_tokens = internal::Token(max_number_of_live_tokens);
-            // Start execution of tasks
-            task::spawn_root_and_wait( *end_counter );
-        } else {
-            // There are no filters, and thus no parallelism is possible.
-            // Just drain the input stream.
-            while( (*filter_list)(NULL) ) 
-                continue;
+        internal::pipeline_cleaner my_pipeline_cleaner(*this);
+        end_of_input = false;
+        input_tokens = internal::Token(max_number_of_live_tokens);
+        if(has_thread_bound_filters) {
+            // release input filter if thread-bound
+            if(filter_list->is_bound()) {
+                filter_list->my_input_buffer->sema_V();
+            }
         }
-    } 
+#if __TBB_TASK_GROUP_CONTEXT            
+        end_counter = new( task::allocate_root(context) ) internal::pipeline_root_task( *this );
+#else
+        end_counter = new( task::allocate_root() ) internal::pipeline_root_task( *this );
+#endif
+        // Start execution of tasks
+        task::spawn_root_and_wait( *end_counter );
+
+        if(has_thread_bound_filters) {
+            for(filter* f = filter_list->next_filter_in_pipeline; f; f=f->next_filter_in_pipeline) {
+                if(f->is_bound()) {
+                    f->my_input_buffer->sema_V(); // wake to end
+                }
+            }
+        }
+    }
 }
 
-#if __TBB_EXCEPTIONS
+#if __TBB_TASK_GROUP_CONTEXT
 void pipeline::run( size_t max_number_of_live_tokens ) {
-    tbb::task_group_context context;
-    run(max_number_of_live_tokens, context);
+    if( filter_list ) {
+        // Construct task group context with the exception propagation mode expected 
+        // by the pipeline caller.
+        uintptr_t ctx_traits = filter_list->my_filter_mode & filter::exact_exception_propagation ? 
+                task_group_context::default_traits :
+                task_group_context::default_traits & ~task_group_context::exact_exception;
+        task_group_context context(task_group_context::bound, ctx_traits);
+        run(max_number_of_live_tokens, context);
+    }
+}
+#endif // __TBB_TASK_GROUP_CONTEXT
+
+bool filter::has_more_work() {
+    __TBB_ASSERT(my_pipeline, NULL);
+    __TBB_ASSERT(my_input_buffer, "has_more_work() called for filter with no input buffer");
+    return (internal::tokendiff_t)(my_pipeline->token_counter - my_input_buffer->low_token) != 0;
 }
-#endif // __TBB_EXCEPTIONS
 
 filter::~filter() {
     if ( (my_filter_mode & version_mask) >= __TBB_PIPELINE_VERSION(3) ) {
-        if ( next_filter_in_pipeline != filter::not_in_pipeline() ) { 
-            __TBB_ASSERT( prev_filter_in_pipeline != filter::not_in_pipeline(), "probably filter list is broken" );
+        if ( next_filter_in_pipeline != filter::not_in_pipeline() )
             my_pipeline->remove_filter(*this);
-        } else 
+        else 
             __TBB_ASSERT( prev_filter_in_pipeline == filter::not_in_pipeline(), "probably filter list is broken" );
     } else {
         __TBB_ASSERT( next_filter_in_pipeline==filter::not_in_pipeline(), "cannot destroy filter that is part of pipeline" );
@@ -631,19 +686,23 @@ thread_bound_filter::result_type thread_bound_filter::try_process_item() {
 }
 
 thread_bound_filter::result_type thread_bound_filter::internal_process_item(bool is_blocking) {
-    internal::task_info info = internal::empty_info;
-    
+    internal::task_info info;
+    info.reset();
+
+    if(my_pipeline && my_pipeline->end_of_input && !has_more_work())
+        return end_of_stream;
+
     if( !prev_filter_in_pipeline ) {
         if( my_pipeline->end_of_input )
             return end_of_stream;
-        while( my_pipeline->input_tokens == 0 ) {
-            if( is_blocking )
-                __TBB_Yield();
-            else
+        while(my_pipeline->input_tokens == 0) {
+            if( !is_blocking ) 
                 return item_not_available;
+            my_input_buffer->sema_P();
         }
         info.my_object = (*this)(info.my_object);
         if( info.my_object ) {
+            __TBB_ASSERT(my_pipeline->input_tokens > 0, "Token failed in thread-bound filter");
             my_pipeline->input_tokens--;
             if( is_ordered() ) {
                 info.my_token = my_pipeline->token_counter;
@@ -655,20 +714,31 @@ thread_bound_filter::result_type thread_bound_filter::internal_process_item(bool
             return end_of_stream; 
         }
     } else { /* this is not an input filter */
-        while( !my_input_buffer->return_item(info, /*advance=*/true) ) {
-            if( my_pipeline->end_of_input && my_input_buffer->low_token == my_pipeline->token_counter )
-                return end_of_stream;
-            if( is_blocking )
-                __TBB_Yield();
-            else
+        while(!my_input_buffer->has_item()) {
+            if(!is_blocking) {
                 return item_not_available;
+            }
+            my_input_buffer->sema_P();
+            if( my_pipeline->end_of_input && !has_more_work()) {
+                return end_of_stream;
+            }
+        }
+        if(!my_input_buffer->return_item(info, /*advance*/true)) {
+            __TBB_ASSERT(0,"return_item failed");
         }
         info.my_object = (*this)(info.my_object);
     }
     if( next_filter_in_pipeline ) {
-        next_filter_in_pipeline->my_input_buffer->put_item(info);
+        if (!next_filter_in_pipeline->my_input_buffer->put_token(info,/*force_put=*/true) ) {
+            __TBB_ASSERT(0, "Couldn't put token after thread-bound buffer");
+        }
     } else {
-        my_pipeline->input_tokens++;
+        size_t ntokens_avail = ++(my_pipeline->input_tokens);
+        if(my_pipeline->filter_list->is_bound()) {
+            if(ntokens_avail == 1) {
+                my_pipeline->filter_list->my_input_buffer->sema_V();
+            }
+        }
     }
 
     return success;
