@@ -1,5 +1,5 @@
 //-------------------------------------------------------------------------
-//    $Id: glossary.h 21358 2010-06-24 23:38:14Z bangerth $
+//    $Id: glossary.h 22974 2010-12-16 18:40:04Z bangerth $
 //    Version: $Name$
 //
 //    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors
@@ -116,8 +116,9 @@
  *
  * For the purpose of a discretization, blocks are the better concept to use
  * since it is not always possible to address individual components of a
- * solution. This is, in particular, the case for non-@ref GlossPrimitive
- * "primitive" elements. Take for instance the solution of the mixed Laplacian
+ * solution. This is, in particular, the case for non-@ref
+ * GlossPrimitive "primitive"
+ * elements. Take for instance the solution of the mixed Laplacian
  * system with the FE_RaviartThomas element (see step-20). There, the first
  * <tt>dim</tt> components are the directional velocities. Since the shape
  * functions are linear combinations of those, these <tt>dim</tt> components
@@ -197,6 +198,33 @@
  * </dd>
  *
  *
+ * <dt class="glossary">@anchor GlossBoundaryForm <b>%Boundary form</b></dt>
+ *
+ * <dd>For a dim-dimensional triangulation in dim-dimensional space,
+ * the boundary form is a vector defined on faces. It is the vector
+ * product of the image of coordinate vectors on the surface of the
+ * unit cell. It is a vector normal to the surface, pointing outwards
+ * and having the length of the surface element.
+ *
+ * A more general definition would be that (at least up to the length
+ * of this vector) it is exactly that vector that is necessary when
+ * considering integration by parts, i.e. equalities of the form
+ * $\int_\Omega \text{div} \vec \phi = -\int_{\partial\Omega} \vec n
+ * \cdot \vec \phi$. Using this definition then also explains what
+ * this vector should be in the case of domains (and corresponding
+ * triangulations) of dimension <code>dim</code> that are embedded in
+ * a space <code>spacedim</code>: in that case, the boundary form is
+ * still a vector defined on the faces of the triangulation; it is
+ * orthogonal to all tangent directions of the boundary and within the
+ * tangent plane of the domain. Note that this is compatible with case
+ * <code>dim==spacedim</code> since there the tangent plane is the
+ * entire space ${\mathbb R}^\text{dim}$.
+ *
+ * In either case, the length of the vector equals the determinant of
+ * the transformation of reference face to the face of the current
+ * cell.  </dd>
+ *
+ *
  * <dt class="glossary">@anchor GlossComponent <b>Component</b></dt>
  *
  * <dd> When considering systems of equations in which the solution is not
@@ -306,6 +334,52 @@
  * </dd>
  *
  *
+ * <dt class="glossary">@anchor GlossDirectionFlag <b>Direction flags</b></dt>
+ *
+ * <dd>The <i>direction flag</i> is used in triangulations embedded in a
+ * higher dimensional space to denote the orientation of cells and make the
+ * manifold oriented. It is accessed using CellAccessor::direction_flag()
+ * and set by the Triangulation class upon creation of a triangulation.
+ *
+ * The flag is necessary to make cases like this work: assume we have a
+ * one-dimensional mesh embedded in a two-dimensional space,
+ *
+ *   @image html direction_flag.png "One dimensional mesh in two dimensions"
+ *
+ * In one dimensional meshes in one dimensional space, we can always make sure
+ * that the location of the left vertex of a cell has a smaller value than the
+ * location of the right vertex. However, if we embed a mesh in a higher
+ * dimensional space, we can no longer do this. For example, the cells in the
+ * mesh above may be described by the following vertex sets: <code>(0,1),
+ * (1,2), (3,2), (4,3), (4,5)</code>. (As a side remark, note that here we
+ * have vertices -- e.g. vertex 2 -- that are the right end points of more
+ * than one cell.) If we define the normal to each cell as that unit vector
+ * that is right perpendicular to the vector that connects the first to the
+ * second vertex of the line, then we would end up with the following picture:
+ *
+ *   @image html direction_flag_normals.png "Normal vectors"
+ *
+ * In other words, this one-dimensional manifold is not oriented. We could in
+ * principle revert the order of vertices when creating such a mesh (though
+ * there are good reasons not to do so, for example because this mesh may have
+ * resulted from extracting the surface mesh of a two dimensional mesh, and we
+ * want to preserve the order of vertices of each line segment because they
+ * currently match the order of vertices of the faces of the 2d cells). An
+ * alternative strategy, chosen in deal.II, is to simply associate with each
+ * cell whether the normal should be the left or right normal to the
+ * cell. (The default is right normals.) In the example above, the flags for
+ * the five cells would be <code>true, true, false, false,
+ * true</code>. Multiplying the right normal with plus or minus one, depending
+ * on the value of the flag on each cell, yields a set of normal vectors that
+ * orient the manifold.
+ *
+ * Similar issues happen with two-dimensional meshes in three space
+ * dimensions. We note that it would not be possible to find consistent
+ * direction flags if the two-dimensional manifold is not orientable; such
+ * manifolds are not currently supported by deal.II.
+ * </dd>
+ *
+ *
  * <dt class="glossary">@anchor GlossDistorted <b>Distorted cells</b></dt>
  *
  * <dd>A <i>distorted cell</i> is a cell for which the mapping from
@@ -421,17 +495,38 @@
  *
  * <dt class="glossary">@anchor distributed_paper
  *                           <b>Distributed computing paper</b></dt>
-
+ *
  * <dd>The "distributed computing paper" is a paper by W. Bangerth,
  * C. Burstedde, T. Heister and M. Kronbichler titled "Algorithms and Data
  * Structures for Massively Parallel Generic Finite Element Codes" that
- * described the implementation of parallel distributed computing in deal.II,
+ * describes the implementation of %parallel distributed computing in deal.II,
  * i.e. computations where not only the linear system is split onto different
- * machines as in, for example, step-18, but also the Triangulation and
+ * machines as in, for example, step-17, but also the Triangulation and
  * DoFHandler objects. In essence, it is a guide to the parallel::distributed
- * namespace.
+ * namespace and the techniques used in step-40.
  *
- * The paper is currently in preparation.
+ * The paper is submitted to the ACM Transactions on Mathematical Software
+ * (ACM TOMS). It is also available as
+ * <a href="http://iamcs.tamu.edu/file_dl.php?type=preprint&preprint_id=237">IAMCS
+ * preprint 2011-187</a>. Its current BiBTex entry is
+ * @code
+TechReport{BBHK11,
+  author = 	 {Wolfgang Bangerth and Carsten Burstedde and Timo Heister
+                  and Martin Kronbichler},
+  title = 	 {Algorithms and data structures for massively parallel generic
+                  adaptive finite element codes},
+  institution =  {Institute for Applied Mathematics and Computational Science,
+                  Texas A&M University},
+  year = 	 2010,
+  number = 	 {2011-187, submitted}
+}
+ * @endcode
+ *
+ * For massively %parallel
+ * computations, deal.II builds on the
+ * <a href="http://www.p4est.org/" target="_top">p4est</a>
+ * library. If you use this functionality, please also cite the
+ * p4est paper listed at their website.
  * </dd>
  *
  *
@@ -607,21 +702,21 @@ Article{JK10,
  * polynomials. Let us give some examples:
  *
  * <table><tr>
- * <th>Element</th>
- * <th>Function space</th>
- * <th>Node values</th></tr>
- * <tr><th>FE_Q, FE_DGQ</th>
- * <td><i>Q<sub>k</sub></i></td>
- * <td>values in support points</td></tr>
- * <tr><th>FE_DGP</th>
- * <td><i>P<sub>k</sub></i></td>
- * <td>moments with respect to Legendre polynomials</td></tr>
- * <tr><th>FE_RaviartThomas (2d)</th>
- * <td><i>Q<sub>k+1,k</sub> x Q<sub>k,k+1</sub></i></td>
- * <td>moments on edges and in the interior</td></tr>
- * <tr><th>FE_RaviartThomasNodal</th>
- * <td><i>Q<sub>k+1,k</sub> x Q<sub>k,k+1</sub></i></td>
- * <td>Gauss points on edges(faces) and anisotropic Gauss points in the interior</td></tr>
+ *   <th>Element</th>
+ *   <th>Function space</th>
+ *   <th>Node values</th></tr>
+ *   <tr><th>FE_Q, FE_DGQ</th>
+ *     <td><i>Q<sub>k</sub></i></td>
+ *     <td>values in support points</td></tr>
+ *   <tr><th>FE_DGP</th>
+ *     <td><i>P<sub>k</sub></i></td>
+ *     <td>moments with respect to Legendre polynomials</td></tr>
+ *   <tr><th>FE_RaviartThomas (2d)</th>
+ *     <td><i>Q<sub>k+1,k</sub> x Q<sub>k,k+1</sub></i></td>
+ *     <td>moments on edges and in the interior</td></tr>
+ *   <tr><th>FE_RaviartThomasNodal</th>
+ *     <td><i>Q<sub>k+1,k</sub> x Q<sub>k,k+1</sub></i></td>
+ *     <td>Gauss points on edges(faces) and anisotropic Gauss points in the interior</td></tr>
  * </table>
  *
  * <dt class="glossary">@anchor GlossPrimitive <b>Primitive finite
@@ -659,16 +754,15 @@ Article{JK10,
  * cell is associated with.
  *
  * For programs that are parallelized based on MPI but where each processor
- * stores the entire triangulation (as in, for example, step-18,
- * subdomain ids are assigned to cells by
+ * stores the entire triangulation (as in, for example, step-18, but not in
+ * step-32), subdomain ids are assigned to cells by
  * partitioning a mesh, and each MPI process then only works on those cells it
- * "owns", i.e. that belong to a subdomain that it is associated with
+ * "owns", i.e. that belong to a subdomain that the processor is associated with
  * (traditionally, this is the case for the subdomain id whose numerical value
  * coincides with the rank of the MPI process within the MPI
  * communicator). Partitioning is typically done using the
  * GridTools::partition() function, but any other method can also be used to
- * do this though most simple ideas will likely lead to less well balanced
- * numbers of degrees of freedom on the various subdomains.
+ * do this.
  *
  * On the other hand, for programs that are parallelized using MPI but
  * where meshes are held distributed across several processors using
@@ -678,7 +772,7 @@ Article{JK10,
  * querying the subdomain id of a cell tells you if the cell is owned
  * by the current processor (i.e. if <code>cell-@>subdomain_id() ==
  * triangulation.parallel::distributed::Triangulation::locally_owned_subdomain()</code>)
- * or by another processor. In the parallel distributed case,
+ * or by another processor. In the %parallel distributed case,
  * subdomain ids are only assigned to cells that the current processor
  * owns as well as the immediately adjacent @ref GlossGhostCell "ghost cells".
  * Cells further away are held on each processor to ensure
