@@ -1,16 +1,9 @@
 /**
- * @page changes_after_6_3_1 Changes after Version 6.3.1
+ * @page changes_after_7_0 Changes after Version 7.0
 
 <p>
 This is the list of changes made after the release of
-deal.II version 6.3.1. It is subdivided into changes
-made to the three sub-libraries <a href="#base">base</a>,
-<a href="#lac">lac</a>, and <a href="#deal.II">deal.II</a>, as well as
-changes to the <a href="#general">general infrastructure,
-documentation, etc</a>.
-</p>
-
-<p>
+deal.II version 7.0.0.
 All entries are signed with the names of the author. Regular
 contributor's names are abbreviated by WB (Wolfgang Bangerth), GK
 (Guido Kanschat), RH (Ralf Hartmann).
@@ -30,7 +23,7 @@ inconvenience this causes.
 </p>
 
 <ol>
-  <li>None so far.
+  <li>
 </ol>
 
 
@@ -38,32 +31,17 @@ inconvenience this causes.
 <h3>General</h3>
 
 <ol>
-  <li>None so far.
+  <li>
 </ol>
 
 
 
-<a name="base"></a>
-<h3>base</h3>
+<a name="specific"></a>
+<h3>Specific improvements</h3>
+
 
 <ol>
-  <li>None so far.
-</ol>
-
-
-<a name="lac"></a>
-<h3>lac</h3>
-
-<ol>
-  <li>None so far.
-</ol>
-
-
-<a name="deal.II"></a>
-<h3>deal.II</h3>
-
-<ol>
-  <li>None so far.
+  <li>
 </ol>
 
 
