@@ -1,8 +1,8 @@
-/* $Id: step-29.cc 15802 2008-02-27 03:08:36Z bangerth $ */
+/* $Id: step-29.cc 22905 2010-12-02 16:45:55Z bangerth $ */
 /* Author: Moritz Allmaras, Texas A&M University, 2007 */
 
 /*                                                                */
-/*    Copyright (C) 2007, 2008 by the deal.II authors and M. Allmaras   */
+/*    Copyright (C) 2007, 2008, 2010 by the deal.II authors and M. Allmaras   */
 /*                                                                */
 /*    This file is subject to QPL and may not be  distributed     */
 /*    without copyright and license information. Please refer     */
@@ -409,17 +409,18 @@ class ComputeIntensity : public DataPostprocessor<dim>
 {
   public:
 
-    void compute_derived_quantities_vector (
-      const std::vector< Vector< double > > &, 
-      const std::vector< std::vector< Tensor< 1, dim > > > &, 
-      const std::vector< std::vector< Tensor< 2, dim > > > &, 
-      const std::vector< Point< dim > > &,
-      std::vector< Vector< double > > &
-    ) const;
+    virtual
+    void
+    compute_derived_quantities_vector (const std::vector< Vector< double > > &uh, 
+				       const std::vector< std::vector< Tensor< 1, dim > > > &duh,
+				       const std::vector< std::vector< Tensor< 2, dim > > > &dduh,
+				       const std::vector< Point< dim > > &normals,
+				       const std::vector<Point<dim> > &evaluation_points,
+				       std::vector< Vector< double > > &computed_quantities) const;
 
-    std::vector<std::string> get_names () const;
-    UpdateFlags              get_needed_update_flags () const;
-    unsigned int             n_output_variables () const;
+    virtual std::vector<std::string> get_names () const;
+    virtual UpdateFlags              get_needed_update_flags () const;
+    virtual unsigned int             n_output_variables () const;
 };
 
 				 // The <code>get_names</code>
@@ -518,6 +519,7 @@ ComputeIntensity<dim>::compute_derived_quantities_vector (
   const std::vector< std::vector< Tensor< 1, dim > > >  & /*duh*/,
   const std::vector< std::vector< Tensor< 2, dim > > >  & /*dduh*/,
   const std::vector< Point< dim > >                     & /*normals*/,
+  const std::vector<Point<dim> >                        & /*evaluation_points*/,
   std::vector< Vector< double > >                       & computed_quantities
 ) const
 {
