@@ -1,4 +1,4 @@
-/* $Id: step-35.cc 21358 2010-06-24 23:38:14Z bangerth $ */
+/* $Id: step-35.cc 21497 2010-07-14 13:50:13Z bangerth $ */
 /* Version: $Name:  $ */
 /*      */
 /*    Copyright (C) 2007, 2008, 2009, 2010 by the deal.II authors */
@@ -241,7 +241,7 @@ namespace RunTimeParameters
 
     prm.enter_subsection ("Data solve velocity");
     {
-      vel_max_iterations = prm.get_double ("max_iterations");
+      vel_max_iterations = prm.get_integer ("max_iterations");
       vel_eps            = prm.get_double ("eps");
       vel_Krylov_size    = prm.get_integer ("Krylov_size");
       vel_off_diagonals  = prm.get_integer ("off_diagonals");
@@ -1033,7 +1033,7 @@ NavierStokesProjection<dim>::run (const bool verbose,
 {
   ConditionalOStream verbose_cout (std::cout, verbose);
 
-  unsigned int n_steps =  (T - t_0)/dt;
+  const unsigned int n_steps =  static_cast<unsigned int>((T - t_0)/dt);
   vel_exact.set_time (2.*dt);
   output_results(1);
   for (unsigned int n = 2; n<=n_steps; ++n)
@@ -1369,8 +1369,8 @@ NavierStokesProjection<dim>::update_pressure (const bool reinit_prec)
 				 // graphical output.
 				 //
 				 // We will not elaborate on this process
-				 // here, but rather refer to step-31,
-				 // where a similar procedure is used
+				 // here, but rather refer to step-31 and
+				 // step-32, where a similar procedure is used
 				 // (and is documented) to create a joint
 				 // DoFHandler object for all variables.
 				 //
