@@ -1,7 +1,7 @@
-/* $Id: step-39.cc 21222 2010-06-17 15:51:46Z kanschat $ */
+/* $Id: step-39.cc 22887 2010-11-30 04:45:36Z bangerth $ */
 /* Author: Guido Kanschat, Texas A&M University, 2009 */
 
-/*    $Id: step-39.cc 21222 2010-06-17 15:51:46Z kanschat $       */
+/*    $Id: step-39.cc 22887 2010-11-30 04:45:36Z bangerth $       */
 /*                                                                */
 /*    Copyright (C) 2010 by the deal.II authors */
 /*                                                                */
@@ -42,6 +42,11 @@
 #include <numerics/mesh_worker_assembler.h>
 #include <numerics/mesh_worker_loop.h>
 
+				 // The include file for local
+				 // integrators associated with the
+				 // Laplacian
+#include <integrators/laplace.h>
+
 				 // Support for multigrid methods
 #include <multigrid/mg_tools.h>
 #include <multigrid/multigrid.h>
@@ -116,47 +121,40 @@ class MatrixIntegrator : public Subscriptor
 
 
 				 // On each cell, we integrate the
-				 // Dirichlet form. All local
-				 // integrations consist of nested
-				 // loops, first over all quadrature
-				 // points and the iner loops over the
-				 // degrees of freedom associated
-				 // with the shape functions.
+				 // Dirichlet form. We use the library
+				 // of ready made integrals in
+				 // LocalIntegrators to avoid writing
+				 // these loops ourselves. Similarly,
+				 // we implement Nitsche boundary
+				 // conditions and the interior
+				 // penalty fluxes between cells.
+				 //
+				 // The boundary und flux terms need a
+				 // penalty parameter, which should be
+				 // adjusted to the cell size and the
+				 // polynomial degree. A safe choice
+				 // of this parameter for constant
+				 // coefficients can be found in
+				 // LocalIntegrators::Laplace::compute_penalty()
+				 // and we use this below.
 template <int dim>
 void MatrixIntegrator<dim>::cell(
   MeshWorker::DoFInfo<dim>& dinfo,
   typename MeshWorker::IntegrationInfo<dim>& info)
 {
-  const FEValuesBase<dim>& fe = info.fe_values();
-  FullMatrix<double>& local_matrix = dinfo.matrix(0,false).matrix;
-  
-  for (unsigned int k=0; k<fe.n_quadrature_points; ++k)
-    for (unsigned int i=0; i<fe.dofs_per_cell; ++i)
-      for (unsigned int j=0; j<fe.dofs_per_cell; ++j)
-	local_matrix(i,j) += (fe.shape_grad(i,k) * fe.shape_grad(j,k))
-			     * fe.JxW(k);
+  LocalIntegrators::Laplace::cell_matrix(dinfo.matrix(0,false).matrix, info.fe_values());
 }
 
-				 // On boundary faces, we use the
-				 // Nitsche boundary condition
+
 template <int dim>
 void MatrixIntegrator<dim>::bdry(
   MeshWorker::DoFInfo<dim>& dinfo,
   typename MeshWorker::IntegrationInfo<dim>& info)
 {
-  const FEValuesBase<dim>& fe = info.fe_values();
-  FullMatrix<double>& local_matrix = dinfo.matrix(0,false).matrix;
-  
-  const unsigned int deg = fe.get_fe().tensor_degree();
-  const double penalty = 2. * deg * (deg+1) * dinfo.face->measure() / dinfo.cell->measure();
-  
-  for (unsigned k=0;k<fe.n_quadrature_points;++k)
-    for (unsigned int i=0; i<fe.dofs_per_cell; ++i)
-      for (unsigned int j=0; j<fe.dofs_per_cell; ++j)
-	local_matrix(i,j) += (fe.shape_value(i,k) * penalty * fe.shape_value(j,k)
-			      - (fe.normal_vector(k) * fe.shape_grad(i,k)) * fe.shape_value(j,k)
-			      - (fe.normal_vector(k) * fe.shape_grad(j,k)) * fe.shape_value(i,k))
-			     * fe.JxW(k);
+  const unsigned int deg = info.fe_values(0).get_fe().tensor_degree();
+  LocalIntegrators::Laplace::nitsche_matrix(
+    dinfo.matrix(0,false).matrix, info.fe_values(0),
+    LocalIntegrators::Laplace::compute_penalty(dinfo, dinfo, deg, deg));
 }
 
 				 // Interior faces use the interior
@@ -168,46 +166,12 @@ void MatrixIntegrator<dim>::face(
   typename MeshWorker::IntegrationInfo<dim>& info1,
   typename MeshWorker::IntegrationInfo<dim>& info2)
 {
-  const FEValuesBase<dim>& fe1 = info1.fe_values();
-  const FEValuesBase<dim>& fe2 = info2.fe_values();
-  FullMatrix<double>& matrix_v1u1 = dinfo1.matrix(0,false).matrix;
-  FullMatrix<double>& matrix_v1u2 = dinfo1.matrix(0,true).matrix;
-  FullMatrix<double>& matrix_v2u1 = dinfo2.matrix(0,true).matrix;
-  FullMatrix<double>& matrix_v2u2 = dinfo2.matrix(0,false).matrix;
-  
-  const unsigned int deg = fe1.get_fe().tensor_degree();
-  double penalty1 = deg * (deg+1) * dinfo1.face->measure() / dinfo1.cell->measure();
-  double penalty2 = deg * (deg+1) * dinfo2.face->measure() / dinfo2.cell->measure();
-  if (dinfo1.cell->has_children() ^ dinfo2.cell->has_children())
-    {
-      Assert (dinfo1.face == dinfo2.face, ExcInternalError());
-      Assert (dinfo1.face->has_children(), ExcInternalError());
-//      Assert (dinfo1.cell->has_children(), ExcInternalError());
-      penalty1 *= 2;
-    }
-const double penalty = penalty1 + penalty2;
-  
-  for (unsigned k=0;k<fe1.n_quadrature_points;++k)
-    for (unsigned int i=0; i<fe1.dofs_per_cell; ++i)
-      for (unsigned int j=0; j<fe1.dofs_per_cell; ++j)
-	{
-	  matrix_v1u1(i,j) += (fe1.shape_value(i,k) * penalty * fe1.shape_value(j,k)
-			       - (fe1.normal_vector(k) * fe1.shape_grad(i,k)) * fe1.shape_value(j,k)
-			       - (fe1.normal_vector(k) * fe1.shape_grad(j,k)) * fe1.shape_value(i,k)
-	  ) * .5 * fe1.JxW(k);
-	  matrix_v1u2(i,j) += (-fe1.shape_value(i,k) * penalty * fe2.shape_value(j,k)
-			       + (fe1.normal_vector(k) * fe1.shape_grad(i,k)) * fe2.shape_value(j,k)
-			       - (fe1.normal_vector(k) * fe2.shape_grad(j,k)) * fe1.shape_value(i,k)
-	  ) * .5 * fe1.JxW(k);
-	  matrix_v2u1(i,j) += (-fe2.shape_value(i,k) * penalty * fe1.shape_value(j,k)
-			       - (fe1.normal_vector(k) * fe2.shape_grad(i,k)) * fe1.shape_value(j,k)
-			       + (fe1.normal_vector(k) * fe1.shape_grad(j,k)) * fe2.shape_value(i,k)
-	  ) * .5 * fe1.JxW(k);
-	  matrix_v2u2(i,j) += (fe2.shape_value(i,k) * penalty * fe2.shape_value(j,k)
-			       + (fe1.normal_vector(k) * fe2.shape_grad(i,k)) * fe2.shape_value(j,k)
-			       + (fe1.normal_vector(k) * fe2.shape_grad(j,k)) * fe2.shape_value(i,k)
-	  ) * .5 * fe1.JxW(k);
-	}
+  const unsigned int deg = info1.fe_values(0).get_fe().tensor_degree();
+  LocalIntegrators::Laplace::ip_matrix(
+    dinfo1.matrix(0,false).matrix, dinfo1.matrix(0,true).matrix, 
+    dinfo2.matrix(0,true).matrix, dinfo2.matrix(0,false).matrix,
+    info1.fe_values(0), info2.fe_values(0),
+    LocalIntegrators::Laplace::compute_penalty(dinfo1, dinfo2, deg, deg));
 }
 
 				 // The second local integrator builds
@@ -383,7 +347,6 @@ class Step39
     void error ();
     double estimate ();
     void solve ();
-    void refine_grid ();
     void output_results (const unsigned int cycle) const;
 
 				     // The member objects related to
@@ -448,6 +411,7 @@ class Step39
 template <int dim>
 Step39<dim>::Step39(const FiniteElement<dim>& fe)
 		:
+                mapping(),
 		fe(fe),
 		mg_dof_handler(triangulation),
 		dof_handler(mg_dof_handler),
@@ -571,25 +535,33 @@ Step39<dim>::assemble_matrix()
 				   // such that they always point to
 				   // the current cell. To this end,
 				   // we need to tell it first, where
-				   // and what to compute,
+				   // and what to compute. Since we
+				   // are not doing anything fancy, we
+				   // can rely on their standard
+				   // choice for quadrature rules.
+				   //
+				   // Since their default update flags
+				   // are minimal, we add what we need
+				   // additionally, namely the values
+				   // and gradients of shape functions
+				   // on all objects (cells, boundary
+				   // and interior faces). Afterwards,
+				   // we are ready to initialize the
+				   // container, which will create all
+				   // necessary FEValuesBase objects
+				   // for integration.
   MeshWorker::IntegrationInfoBox<dim> info_box;
-				   // namely, which quadrature
-				   // formulas to use and
-  const unsigned int n_gauss_points = dof_handler.get_fe().tensor_degree()+1;
-  info_box.initialize_gauss_quadrature(n_gauss_points, n_gauss_points, n_gauss_points);
-				   // which values to update in these
-				   // points. We call
-				   // <tt>initialize_update_flags</tt>
-				   // first in order to set default
-				   // values. Then, we add what we
-				   // need additionally.
-  info_box.initialize_update_flags();
   UpdateFlags update_flags = update_values | update_gradients;
-  info_box.add_update_flags(update_flags, true, true, true, true);
+  info_box.add_update_flags_all(update_flags);
   info_box.initialize(fe, mapping);
 
 				   // This is the object into which we
-				   // integrate local data.
+				   // integrate local data. It is
+				   // filled by the local integration
+				   // routines in MatrixIntegrator and
+				   // then used by the assembler to
+				   // distribute the information into
+				   // the global matrix.
   MeshWorker::DoFInfo<dim> dof_info(dof_handler);
 
 				   // Finally, we need an object that
@@ -627,11 +599,8 @@ void
 Step39<dim>::assemble_mg_matrix()
 {
   MeshWorker::IntegrationInfoBox<dim> info_box;
-  const unsigned int n_gauss_points = mg_dof_handler.get_fe().tensor_degree()+1;
-  info_box.initialize_gauss_quadrature(n_gauss_points, n_gauss_points, n_gauss_points);
-  info_box.initialize_update_flags();
   UpdateFlags update_flags = update_values | update_gradients;
-  info_box.add_update_flags(update_flags, true, true, true, true);
+  info_box.add_update_flags_all(update_flags);
   info_box.initialize(fe, mapping);
 
   MeshWorker::DoFInfo<dim> dof_info(mg_dof_handler);
@@ -673,11 +642,8 @@ void
 Step39<dim>::assemble_right_hand_side()
 {
   MeshWorker::IntegrationInfoBox<dim> info_box;
-  const unsigned int n_gauss_points = dof_handler.get_fe().tensor_degree()+1;
-  info_box.initialize_gauss_quadrature(n_gauss_points, n_gauss_points+1, n_gauss_points);
-  info_box.initialize_update_flags();
   UpdateFlags update_flags = update_quadrature_points | update_values | update_gradients;
-  info_box.add_update_flags(update_flags, true, true, true, true);
+  info_box.add_update_flags_all(update_flags);
   info_box.initialize(fe, mapping);
   
   MeshWorker::DoFInfo<dim> dof_info(dof_handler);
@@ -721,7 +687,7 @@ Step39<dim>::solve()
 				   // The solver of choice is
 				   // conjugate gradient.
   SolverControl control(1000, 1.e-12);
-  SolverCG<Vector<double> > cg(control);
+  SolverCG<Vector<double> > solver(control);
 
 				   // Now we are setting up the
 				   // components of the multilevel
@@ -797,7 +763,7 @@ Step39<dim>::solve()
     MGTransferPrebuilt<Vector<double> > >
     preconditioner(mg_dof_handler, mg, mg_transfer);
 				   // and use it to solve the system.
-  cg.solve(matrix, solution, right_hand_side, preconditioner);
+  solver.solve(matrix, solution, right_hand_side, preconditioner);
 }
 
 				 // Here we compare our finite element
@@ -856,11 +822,18 @@ Step39<dim>::estimate()
 				   // Then, we tell the Meshworker::VectorSelector
 				   // for cells, that we need the
 				   // second derivatives of this
-				   // solution (to compute the Laplacian).
+				   // solution (to compute the
+				   // Laplacian). Therefore, the
+				   // boolean arguments selecting
+				   // function values and first
+				   // derivatives a false, only the
+				   // last one selecting second
+				   // derivatives is true.
   info_box.cell_selector.add("solution", false, false, true);
 				   // On interior and boundary faces,
 				   // we need the function values and
-				   // the first derivatives.
+				   // the first derivatives, but not
+				   // second derivatives.
   info_box.boundary_selector.add("solution", true, true, false);
   info_box.face_selector.add("solution", true, true, false);
 
@@ -869,8 +842,7 @@ Step39<dim>::estimate()
 				   // update flags are already
 				   // adjusted to the values and
 				   // derivatives we requested above.
-  info_box.initialize_update_flags();
-  info_box.add_update_flags(update_quadrature_points, false, true, false, false);
+  info_box.add_update_flags_boundary(update_quadrature_points);
   info_box.initialize(fe, mapping, solution_data);
   
   MeshWorker::DoFInfo<dim> dof_info(dof_handler);
