@@ -1,9 +1,9 @@
-/* $Id: step-4.cc 21235 2010-06-20 17:53:33Z kanschat $ */
+/* $Id: step-4.cc 23105 2011-01-03 01:22:08Z bangerth $ */
 /* Author: Wolfgang Bangerth, University of Heidelberg, 1999 */
 
-/*    $Id: step-4.cc 21235 2010-06-20 17:53:33Z kanschat $       */
+/*    $Id: step-4.cc 23105 2011-01-03 01:22:08Z bangerth $       */
 /*                                                                */
-/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 by the deal.II authors */
+/*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by the deal.II authors */
 /*                                                                */
 /*    This file is subject to QPL and may not be  distributed     */
 /*    without copyright and license information. Please refer     */
@@ -108,38 +108,37 @@ class LaplaceProblem
 				 // dim-dimensional space variable, so we
 				 // declare them as templates as well.
 				 //
-				 // Each of these classes is derived
-				 // from a common, abstract base class
-				 // Function, which declares the
-				 // common interface which all
-				 // functions have to follow. In
-				 // particular, concrete classes have
-				 // to overload the `value' function,
-				 // which takes a point in
-				 // dim-dimensional space as
-				 // parameters and shall return the
-				 // value at that point as a `double'
-				 // variable.
+				 // Each of these classes is derived from a
+				 // common, abstract base class Function,
+				 // which declares the common interface which
+				 // all functions have to follow. In
+				 // particular, concrete classes have to
+				 // overload the <code>value</code> function,
+				 // which takes a point in dim-dimensional
+				 // space as parameters and shall return the
+				 // value at that point as a
+				 // <code>double</code> variable.
 				 //
-				 // The `value' function takes a second
-				 // argument, which we have here named
-				 // `component': This is only meant for vector
-				 // valued functions, where you may want to
-				 // access a certain component of the vector
-				 // at the point `p'. However, our functions
-				 // are scalar, so we need not worry about
-				 // this parameter and we will not use it in
-				 // the implementation of the
-				 // functions. Inside the library's header
-				 // files, the Function base class's
-				 // declaration of the `value' function has a
-				 // default value of zero for the component,
-				 // so we will access the `value' function of
-				 // the right hand side with only one
-				 // parameter, namely the point where we want
-				 // to evaluate the function. A value for the
-				 // component can then simply be omitted for
-				 // scalar functions.
+				 // The <code>value</code> function takes a
+				 // second argument, which we have here named
+				 // <code>component</code>: This is only meant
+				 // for vector valued functions, where you may
+				 // want to access a certain component of the
+				 // vector at the point
+				 // <code>p</code>. However, our functions are
+				 // scalar, so we need not worry about this
+				 // parameter and we will not use it in the
+				 // implementation of the functions. Inside
+				 // the library's header files, the Function
+				 // base class's declaration of the
+				 // <code>value</code> function has a default
+				 // value of zero for the component, so we
+				 // will access the <code>value</code>
+				 // function of the right hand side with only
+				 // one parameter, namely the point where we
+				 // want to evaluate the function. A value for
+				 // the component can then simply be omitted
+				 // for scalar functions.
 				 //
 				 // Note that the C++ language forces
 				 // us to declare and define a
@@ -204,7 +203,7 @@ class BoundaryValues : public Function<dim>
 				 // The last thing to note is that a
 				 // <code>Point@<dim@></code> denotes a point in
 				 // dim-dimensionsal space, and its individual
-				 // components (i.e. `x', `y',
+				 // components (i.e. $x$, $y$,
 				 // ... coordinates) can be accessed using the
 				 // () operator (in fact, the [] operator will
 				 // work just as well) with indices starting
@@ -243,36 +242,41 @@ double BoundaryValues<dim>::value (const Point<dim> &p,
                                  // template that makes use of the functions
                                  // above. As before, we will write everything
                                  // as templates that have a formal parameter
-                                 // <code>dim</code> that we assume unknown at the time
-                                 // we define the template functions. Only
-                                 // later, the compiler will find a
-                                 // declaration of <code>LaplaceProblem@<2@></code> (in
-                                 // the <code>main</code> function, actually) and
-                                 // compile the entire class with <code>dim</code>
-                                 // replaced by 2, a process referred to as
-                                 // `instantiation of a template'. When doing
-                                 // so, it will also replace instances of
+                                 // <code>dim</code> that we assume unknown at
+                                 // the time we define the template
+                                 // functions. Only later, the compiler will
+                                 // find a declaration of
+                                 // <code>LaplaceProblem@<2@></code> (in the
+                                 // <code>main</code> function, actually) and
+                                 // compile the entire class with
+                                 // <code>dim</code> replaced by 2, a process
+                                 // referred to as `instantiation of a
+                                 // template'. When doing so, it will also
+                                 // replace instances of
                                  // <code>RightHandSide@<dim@></code> by
-                                 // <code>RightHandSide@<2@></code> and instantiate the
-                                 // latter class from the class template.
+                                 // <code>RightHandSide@<2@></code> and
+                                 // instantiate the latter class from the
+                                 // class template.
                                  //
                                  // In fact, the compiler will also find a
-                                 // declaration <code>LaplaceProblem@<3@></code> in
-                                 // <code>main()</code>. This will cause it to again go
-                                 // back to the general
-                                 // <code>LaplaceProblem@<dim@></code> template, replace
-                                 // all occurrences of <code>dim</code>, this time by
-                                 // 3, and compile the class a second
-                                 // time. Note that the two instantiations
+                                 // declaration
+                                 // <code>LaplaceProblem@<3@></code> in
+                                 // <code>main()</code>. This will cause it to
+                                 // again go back to the general
+                                 // <code>LaplaceProblem@<dim@></code>
+                                 // template, replace all occurrences of
+                                 // <code>dim</code>, this time by 3, and
+                                 // compile the class a second time. Note that
+                                 // the two instantiations
                                  // <code>LaplaceProblem@<2@></code> and
-                                 // <code>LaplaceProblem@<3@></code> are completely
-                                 // independent classes; their only common
-                                 // feature is that they are both instantiated
-                                 // from the same general template, but they
-                                 // are not convertible into each other, for
-                                 // example, and share no code (both
-                                 // instantiations are compiled completely
-                                 // independently).
+                                 // <code>LaplaceProblem@<3@></code> are
+                                 // completely independent classes; their only
+                                 // common feature is that they are both
+                                 // instantiated from the same general
+                                 // template, but they are not convertible
+                                 // into each other, for example, and share no
+                                 // code (both instantiations are compiled
+                                 // completely independently).
 
 
                                  // @sect4{LaplaceProblem::LaplaceProblem}
@@ -285,7 +289,8 @@ double BoundaryValues<dim>::value (const Point<dim> &p,
 				 // triangulation just as in the previous
 				 // example program, step-3:
 template <int dim>
-LaplaceProblem<dim>::LaplaceProblem () :
+LaplaceProblem<dim>::LaplaceProblem ()
+		:
                 fe (1),
 		dof_handler (triangulation)
 {}
@@ -293,23 +298,20 @@ LaplaceProblem<dim>::LaplaceProblem () :
 
                                  // @sect4{LaplaceProblem::make_grid_and_dofs}
 
-				 // Grid creation is something
-				 // inherently dimension
-				 // dependent. However, as long as the
-				 // domains are sufficiently similar
-				 // in 2D or 3D, the library can
-				 // abstract for you. In our case, we
-				 // would like to again solve on the
-				 // square [-1,1]x[-1,1] in 2D, or on
-				 // the cube [-1,1]x[-1,1]x[-1,1] in
-				 // 3D; both can be termed
-				 // <code>hyper_cube</code>, so we may use the
-				 // same function in whatever
-				 // dimension we are. Of course, the
-				 // functions that create a hypercube
-				 // in two and three dimensions are
-				 // very much different, but that is
-				 // something you need not care
+				 // Grid creation is something inherently
+				 // dimension dependent. However, as long as
+				 // the domains are sufficiently similar in 2D
+				 // or 3D, the library can abstract for
+				 // you. In our case, we would like to again
+				 // solve on the square $[-1,1]\times [-1,1]$
+				 // in 2D, or on the cube $[-1,1] \times
+				 // [-1,1] \times [-1,1]$ in 3D; both can be
+				 // termed <code>hyper_cube</code>, so we may
+				 // use the same function in whatever
+				 // dimension we are. Of course, the functions
+				 // that create a hypercube in two and three
+				 // dimensions are very much different, but
+				 // that is something you need not care
 				 // about. Let the library handle the
 				 // difficult things.
 				 //
@@ -320,12 +322,12 @@ LaplaceProblem<dim>::LaplaceProblem () :
 				 // either. This function therefore looks
 				 // exactly like in the previous example,
 				 // although it performs actions that in their
-				 // details are quite different if <code>dim</code>
-				 // happens to be 3. The only significant
-				 // difference from a user's perspective is
-				 // the number of cells resulting, which is
-				 // much higher in three than in two space
-				 // dimensions!
+				 // details are quite different if
+				 // <code>dim</code> happens to be 3. The only
+				 // significant difference from a user's
+				 // perspective is the number of cells
+				 // resulting, which is much higher in three
+				 // than in two space dimensions!
 template <int dim>
 void LaplaceProblem<dim>::make_grid_and_dofs ()
 {
@@ -408,11 +410,14 @@ void LaplaceProblem<dim>::assemble_system ()
 				   // quadrature points on the cell we are
 				   // presently on (previously, we only
 				   // required values and gradients of the
-				   // shape function from the <code>FEValues</code>
-				   // object, as well as the quadrature
-				   // weights, <code>JxW</code>). We can tell the
-				   // <code>FEValues</code> object to do for us by also
-				   // giving it the <code>update_quadrature_points</code> flag:
+				   // shape function from the
+				   // <code>FEValues</code> object, as well as
+				   // the quadrature weights,
+				   // <code>JxW</code>). We can tell the
+				   // <code>FEValues</code> object to do for
+				   // us by also giving it the
+				   // <code>update_quadrature_points</code>
+				   // flag:
   FEValues<dim> fe_values (fe, quadrature_formula, 
 			   update_values   | update_gradients |
                            update_quadrature_points | update_JxW_values);
@@ -438,15 +443,18 @@ void LaplaceProblem<dim>::assemble_system ()
 				   // Note, that a cell is a quadrilateral in
 				   // two space dimensions, but a hexahedron
 				   // in 3D. In fact, the
-				   // <code>active_cell_iterator</code> data type is
-				   // something different, depending on the
-				   // dimension we are in, but to the outside
-				   // world they look alike and you will
-				   // probably never see a difference although
-				   // the classes that this typedef stands for
-				   // are in fact completely unrelated:
-  typename DoFHandler<dim>::active_cell_iterator cell = dof_handler.begin_active(),
-						 endc = dof_handler.end();
+				   // <code>active_cell_iterator</code> data
+				   // type is something different, depending
+				   // on the dimension we are in, but to the
+				   // outside world they look alike and you
+				   // will probably never see a difference
+				   // although the classes that this typedef
+				   // stands for are in fact completely
+				   // unrelated:
+  typename DoFHandler<dim>::active_cell_iterator
+    cell = dof_handler.begin_active(),
+    endc = dof_handler.end();
+  
   for (; cell!=endc; ++cell)
     {
       fe_values.reinit (cell);
@@ -490,18 +498,18 @@ void LaplaceProblem<dim>::assemble_system ()
                                        // As a final remark to these loops:
                                        // when we assemble the local
                                        // contributions into
-				       // <code>cell_matrix(i,j)</code>,
-				       // we have to multiply
-                                       // the gradients of shape functions $i$
-                                       // and $j$ at point q_point and multiply
-                                       // it with the scalar weights JxW. This
-                                       // is what actually happens:
+                                       // <code>cell_matrix(i,j)</code>, we
+                                       // have to multiply the gradients of
+                                       // shape functions $i$ and $j$ at point
+                                       // q_point and multiply it with the
+                                       // scalar weights JxW. This is what
+                                       // actually happens:
                                        // <code>fe_values.shape_grad(i,q_point)</code>
-                                       // returns a <code>dim</code> dimensional
-                                       // vector, represented by a
-                                       // <code>Tensor@<1,dim@></code> object, and the
-                                       // operator* that multiplies it with
-                                       // the result of
+                                       // returns a <code>dim</code>
+                                       // dimensional vector, represented by a
+                                       // <code>Tensor@<1,dim@></code> object,
+                                       // and the operator* that multiplies it
+                                       // with the result of
                                        // <code>fe_values.shape_grad(j,q_point)</code>
                                        // makes sure that the <code>dim</code>
                                        // components of the two vectors are
@@ -510,13 +518,14 @@ void LaplaceProblem<dim>::assemble_system ()
                                        // that then is multiplied with the
                                        // weights. Internally, this operator*
                                        // makes sure that this happens
-                                       // correctly for all <code>dim</code> components
-                                       // of the vectors, whether <code>dim</code> be
-                                       // 2, 3, or any other space dimension;
-                                       // from a user's perspective, this is
-                                       // not something worth bothering with,
-                                       // however, making things a lot simpler
-                                       // if one wants to write code dimension
+                                       // correctly for all <code>dim</code>
+                                       // components of the vectors, whether
+                                       // <code>dim</code> be 2, 3, or any
+                                       // other space dimension; from a user's
+                                       // perspective, this is not something
+                                       // worth bothering with, however,
+                                       // making things a lot simpler if one
+                                       // wants to write code dimension
                                        // independently.
       
 				       // With the local systems assembled,
@@ -539,14 +548,14 @@ void LaplaceProblem<dim>::assemble_system ()
   
 				   // As the final step in this function, we
 				   // wanted to have non-homogeneous boundary
-				   // values in this example, unlike the
-				   // one before. This is a simple task, we
-				   // only have to replace the
-				   // <code>ZeroFunction</code> used there by an object
-				   // of the class which describes the
-				   // boundary values we would like to use
-				   // (i.e. the <code>BoundaryValues</code> class
-				   // declared above):
+				   // values in this example, unlike the one
+				   // before. This is a simple task, we only
+				   // have to replace the
+				   // <code>ZeroFunction</code> used there by
+				   // an object of the class which describes
+				   // the boundary values we would like to use
+				   // (i.e. the <code>BoundaryValues</code>
+				   // class declared above):
   std::map<unsigned int,double> boundary_values;
   VectorTools::interpolate_boundary_values (dof_handler,
 					    0,
@@ -572,9 +581,9 @@ template <int dim>
 void LaplaceProblem<dim>::solve () 
 {
   SolverControl           solver_control (1000, 1e-12);
-  SolverCG<>              cg (solver_control);
-  cg.solve (system_matrix, solution, system_rhs,
-	    PreconditionIdentity());
+  SolverCG<>              solver (solver_control);
+  solver.solve (system_matrix, solution, system_rhs,
+		PreconditionIdentity());
 
 				   // We have made one addition,
 				   // though: since we suppress output
@@ -612,9 +621,9 @@ void LaplaceProblem<dim>::solve ()
                                  // dimension in the filename to generate
                                  // distinct filenames for each run (in a
                                  // better program, one would check whether
-                                 // `dim' can have other values than 2 or 3,
-                                 // but we neglect this here for the sake of
-                                 // brevity).
+                                 // <code>dim</code> can have other values
+                                 // than 2 or 3, but we neglect this here for
+                                 // the sake of brevity).
 template <int dim>
 void LaplaceProblem<dim>::output_results () const
 {
@@ -658,11 +667,11 @@ void LaplaceProblem<dim>::run ()
 				 // looks mostly like in step-3, but if you
 				 // look at the code below, note how we first
 				 // create a variable of type
-				 // <code>LaplaceProblem@<2@></code> (forcing the
-				 // compiler to compile the class template
-				 // with <code>dim</code> replaced by <code>2</code>) and run a
-				 // 2d simulation, and then we do the whole
-				 // thing over in 3d.
+				 // <code>LaplaceProblem@<2@></code> (forcing
+				 // the compiler to compile the class template
+				 // with <code>dim</code> replaced by
+				 // <code>2</code>) and run a 2d simulation,
+				 // and then we do the whole thing over in 3d.
 				 //
 				 // In practice, this is probably not what you
 				 // would do very frequently (you probably
@@ -685,16 +694,16 @@ void LaplaceProblem<dim>::run ()
 				 //
 				 // Each of the two blocks is enclosed in
 				 // braces to make sure that the
-				 // <code>laplace_problem_2d</code> variable goes out
-				 // of scope (and releases the memory it
-				 // holds) before we move on to allocate
+				 // <code>laplace_problem_2d</code> variable
+				 // goes out of scope (and releases the memory
+				 // it holds) before we move on to allocate
 				 // memory for the 3d case. Without the
 				 // additional braces, the
-				 // <code>laplace_problem_2d</code> variable would only
-				 // be destroyed at the end of the function,
-				 // i.e. after running the 3d problem, and
-				 // would needlessly hog memory while the 3d
-				 // run could actually use it.
+				 // <code>laplace_problem_2d</code> variable
+				 // would only be destroyed at the end of the
+				 // function, i.e. after running the 3d
+				 // problem, and would needlessly hog memory
+				 // while the 3d run could actually use it.
                                  //
                                  // Finally, the first line of the function is
                                  // used to suppress some output.  Remember
@@ -703,7 +712,8 @@ void LaplaceProblem<dim>::run ()
                                  // starting residual and the number of the
                                  // iteration where convergence was
                                  // detected. This can be suppressed through
-                                 // the <code>deallog.depth_console(0)</code> call.
+                                 // the <code>deallog.depth_console(0)</code>
+                                 // call.
                                  //
                                  // The rationale here is the following: the
                                  // deallog (i.e. deal-log, not de-allog)
