@@ -1,7 +1,7 @@
-/* $Id: step-5.cc 21235 2010-06-20 17:53:33Z kanschat $ */
+/* $Id: step-5.cc 22321 2010-10-12 21:53:08Z kanschat $ */
 /* Author: Wolfgang Bangerth, University of Heidelberg, 1999 */
 
-/*    $Id: step-5.cc 21235 2010-06-20 17:53:33Z kanschat $       */
+/*    $Id: step-5.cc 22321 2010-10-12 21:53:08Z kanschat $       */
 /*                                                                */
 /*    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2010 by the deal.II authors */
 /*                                                                */
@@ -636,13 +636,13 @@ template <int dim>
 void LaplaceProblem<dim>::solve ()
 {
   SolverControl           solver_control (1000, 1e-12);
-  SolverCG<>              cg (solver_control);
+  SolverCG<>              solver (solver_control);
 
   PreconditionSSOR<> preconditioner;
   preconditioner.initialize(system_matrix, 1.2);
 
-  cg.solve (system_matrix, solution, system_rhs,
-	    preconditioner);
+  solver.solve (system_matrix, solution, system_rhs,
+		preconditioner);
 
   std::cout << "   " << solver_control.last_step()
 	    << " CG iterations needed to obtain convergence."
